﻿// hmcBuettenrand.jsx
// Copyright © by Manfred Clemens, erstellt: 31.03.2013, letzte Änderung: 27.04.2013
// *********************************************************************************
/* 
  PROGRAMMBESCHREIBUNG:
  Das Programm erstellt ein neues Dokument mit einem unregelmäßigen, fasrig-
  ausgedünnten Dokumentenrand, der auch als Büttenrand bezeichnet werden kann, z. B. für
  die Verwendung von alten Fotos, Ansichtskarten, Urkunden oder anderen Papierprodukten.
  Die schwarze Hintergrundebene dient nur zur besseren optischen Darstellung des 
  Büttenrandes und kann ggf. entfernt werden.
  
  HINWEISE:
  Die Kante und teilweise die Mitte des Büttenrandpapiers, kann mit dem 
  'Nachbelichter-Werkzeug [O], mit einer Belichtung von ca. 25% und unterschiedlichen
  Pinselgrößen leicht nachbelichtet werden, hierdurch entsteht der Eindruck eines 
  alten handgeschöpften Büttenpapiers.
  Die Datei wurde mit ExtendScript Toolkit CS6 erstellt und getestet. 

  INSTALLATION IN PHOTOSHOP & SKRIPTAUFRUF:
  Die Datei 'hmcBuettenrand.jsx' in das Verzeichnis
  'c:\Programme\Adobe\Adobe Photoshop CS6\Presets\Scripts' kopieren und PS neu
  starten, danach den Menübefehl
  'Menü > Datei > Skripten >  hmcBuettenrand' aufrufen und anwenden.
*/  
#target Photoshop
#strict on

// Benutzer Einstellungen/Vorgaben
var scriptName = "hmcBüttenrand";
var scriptVersion = " 1.1";

var picWidth = 600;
var picHight = 450;
var resolution = 300;

// Vordergrundfarbe definieren und setzen
var fgColor = new SolidColor();
fgColor.rgb["hexValue"] = "ffffff"; // white 
app.foregroundColor = fgColor

// Hintergrundfarbe definieren und setzen
var bgColor = new SolidColor();
bgColor.rgb["hexValue"] = "000000"; // black 
app.backgroundColor = bgColor

// Büttenrand-Papierfarbe definieren
var paperColor = new SolidColor();
paperColor.rgb["hexValue"] = "cac2a8";

try {
  
  // Filterfunktion: Filter > Malfilter > Spritzer
  function spritzer(radius, glatt) {
  var id103 = charIDToTypeID( "Spt " );
    var desc25 = new ActionDescriptor();
    var id104 = charIDToTypeID( "GEfk" );
    var id105 = charIDToTypeID( "GEft" );
    var id106 = charIDToTypeID( "Spt " );
    desc25.putEnumerated( id104, id105, id106 );
    var id107 = charIDToTypeID( "SprR" );
    desc25.putInteger( id107, radius );
    var id108 = charIDToTypeID( "Smth" );
    desc25.putInteger( id108, glatt );
    executeAction( id103, desc25, DialogModes.NO );
  }

  // Farbbereichs-Auswahl
  function selectColorRange(scObj){
    var desc = new ActionDescriptor();
    desc.putInteger( charIDToTypeID( "Fzns" ), 0 );
    var cDesc = new ActionDescriptor();
        cDesc.putDouble( charIDToTypeID( "Rd  " ), scObj.rgb.red);
        cDesc.putDouble( charIDToTypeID( "Grn " ), scObj.rgb.green);
        cDesc.putDouble( charIDToTypeID( "Bl  " ), scObj.rgb.blue );
    desc.putObject( charIDToTypeID( "Mnm " ), charIDToTypeID( "RGBC" ), cDesc );
    desc.putObject( charIDToTypeID( "Mxm " ), charIDToTypeID( "RGBC" ), cDesc );
    executeAction( charIDToTypeID( "ClrR" ), desc, DialogModes.NO );
  }

  // Check ob numerische Zahleneingabe
  function IsNumeric(sText) {
    var ValidChars = "0123456789";
    var IsNumber = true;
    var Char;
    for (i = 0; i < sText.length && IsNumber == true; i++) { 
      Char = sText.charAt(i); 
      if (ValidChars.indexOf(Char) == -1) {
        IsNumber = false;
        alert("Keine gültige numerische Zahleneingabe!", "Fehler", Error);
      }
    }
    return IsNumber;
  }

        // Original Maßeinheit speichern und die gewünschte Maßeinheit setzen
        var defaultRulerUnits = app.preferences.rulerUnits; 
        if (defaultRulerUnits != Units.PIXELS) {
          app.preferences.rulerUnits = Units.PIXELS; 
        }


  function CreateDialog() { 
    this.windowRef = null; 

    // **************************************************************************
    // ***                        Dialog hinzufügen                           ***
    // **************************************************************************

    var dlg = new Window("dialog", scriptName + scriptVersion ,[100,100,380,445]); 
    this.windowRef = dlg;
  
    // **************************************************************************
    // ***                        Panel hinzufügen                            ***
    // **************************************************************************
    dlg.infoPanel = dlg.add("panel", [10,10,270,155], "Beschreibung"); 
    with (dlg.infoPanel) {
      // StaticText hinzufügen
      var txtInfo = "Das Programm erstellt ein neues Dokument \n" +
                    "mit einem unregelmäßigen, fasrig-ausge- \n" +
                    "dünnten Dokumentenrand, der auch als \n" +
                    "Büttenrand bezeichnet werden kann, z. B. \n" +
                    "für die Verwendung von alten Fotos,  \n" +
                    "Ansichtskarten, Urkunden oder anderen  \n" +
                    "Papierprodukten.";
      infoTitel = add('statictext',[10,10,260,90], txtInfo , {multiline:true});
    }

    // **************************************************************************
    // ***                        Panel hinzufügen                            ***
    // **************************************************************************
    dlg.docPanel = dlg.add("panel", [10,170,270,250], "Neues Dokument"); 
    with (dlg.docPanel) {
      picWidth1 = add('statictext',[10,10,45,15], "Breite:");
      picWidth2 = add('edittext',[75,10,115,30], picWidth);
      picWidth3 = add('statictext',[120,10,135,15], "px");
      picHight1 = add('statictext',[150,10,190,15], "Höhe:");
      picHight2 = add('edittext',[190,10,230,30], picHight); 
      picHight3 = add('statictext',[235,10,250,15], "px");
      resolution1 = add('statictext',[10,40,70,65], "Auflösung:");
      resolution2 = add('edittext',[75,40,115,60], resolution);
      resolution3 = add('statictext',[120,40,180,60], "Pixel/Zoll");
    }
  
    // **************************************************************************
    // ***                        Panel hinzufügen                            ***
    // **************************************************************************
    dlg.btnPanel = dlg.add("panel", [10,270,270,320], "");
    with (dlg.btnPanel) {
      // Button's hinzufügen
      okBtn = dlg.btnPanel.add("button", [10,10,110,35], "OK");
      cancelBtn = dlg.btnPanel.add("button", [145,10,245,35], "Abbruch");

      // ************************************************************************
      // ***                  Klick-Ereignise auswerten                       ***
      // ************************************************************************
      // OK-Button
      okBtn.onClick = function() {
        
          // Eingabewerte überprüfen
          if (IsNumeric(picWidth2.text) && IsNumeric(picHight2.text) 
                                        && IsNumeric(resolution2.text)) {
                                          
            // Eingabewerte an Variable übergeben
            picWidth = picWidth2.text;
            picHight = picHight2.text;
            resolution = resolution2.text;
           
          
          // neues Dokument erzeugen mit schwarzem Hintergrund
          app.documents.add(picWidth * 1.2, picHight * 1.2, resolution, "hmcBuettenrand", 
                            NewDocumentMode.RGB, DocumentFill.BACKGROUNDCOLOR);
        
        with (app.activeDocument) {
          // eine Hintergund-Ebene in eine normale Ebene wandeln
          activeLayer.isBackgroundLayer = false;
          activeLayer.name = "Buettenrand";
                      
          // Rahmen festlegen, selektieren, mit weißer Farbe füllen, Auswahl wieder aufheben
          selectionBounds = new Array (
            new Array(picWidth / 10, picHight / 10),
            new Array(picWidth * 1.1, picHight / 10),
            new Array(picWidth * 1.1, picHight * 1.1),
            new Array(picWidth / 10, picHight * 1.1)
          );
        
          selection.select(selectionBounds);
          selection.fill (foregroundColor, ColorBlendMode.NORMAL);
          selection.deselect();
        
          // mehrmaliger Funktionsaufruf: Filter > Malfilter > Spritzer
          spritzer( 10, 15);
          spritzer( 10, 15);
          spritzer( 10, 15);
          spritzer( 10, 15);
        
          // Auswahl, weißer Farbbereich
          selectColorRange(fgColor);
        
          // alte Papierfarbe          
          selection.fill(paperColor);
        
          // Auswahl umkehren, löschen und die Auswahl aufheben
          selection.invert();
          selection.clear();
          selection.deselect();

          // neuen Ebene erzeugen, mit schwarzer Farbe füllen, und an die unterste 
          // Stelle in der Ebenenpalette schieben, sie dient nur zur besseren Optik
          var newLay = artLayers.add();
          newLay.name = "Hintergrund schwarz";
          // Ebene an die unterste Position schieben und mit schwarzer Farbe füllen
          activeLayer.move(activeDocument, ElementPlacement.PLACEATEND);
          selection.fill(bgColor);
          
          // Ebene Buettenrand aktivieren
          activeLayer = artLayers[0];

}
        dlg.close();
      }
      }
    
      // Abbruch-Button
      cancelBtn.onClick = function() {
        dlg.close();
      }
    }
  
  
    // **************************************************************************
    // ***                 Dialog anzeigen und plazieren                      ***
    // **************************************************************************
    //dlg.center();
    dlg.frameLocation = [600,100];              // Dialog-Fenster plazieren
    dlg.show();                                 // Dialog-Fenster anzeigen
    //return true;



    // Original PS-Einstellungen wieder zurücksetzen
    function Cleanup() {
      app.preferences.rulerUnits = defaultRulerUnits;
      app.preferences.typeUnits = defaultTypeUnits;
      app.DisplayDialogs = defaultDisplayDialogs;
      return;
    }
}

// ******************************************************************************
// ***                    Hauptprogramm - Programmstart                       ***
// ******************************************************************************

CreateDialog(); // Hauptprogramm - Programmstart

app.preferences.rulerUnits = defaultRulerUnits; // urspr. Maßeinheit wieder setzen

// Speicher wieder freimachen
scriptName = null;
scriptVersion = null;
picWidth = null;
picHight = null;
resolution = null;
fgColor = null;
bgColor = null;
paperColor = null;

} // try Ende
catch (e) {
  alert (e.description, "Fehler!");
} // EOF